#!/usr/bin/env python
"""
Python Package Scaffold Builder

Usage:
    create-pypkg -h|--help
    create-pypkg --version
    create-pypkg [--debug|--info] [--module=<name>] <path>

Options:
    -h, --help          Print help and exit
    --version           Print version and exit
    --debug, --info     Execute a command with debug|info messages
    --module=<name>     Specify a module name

Arguments:
    <path>              Path to a directory for a new repository directory
"""

import logging
import os
import re
from pathlib import Path

from docopt import docopt

from .. import __version__
from .util import (fetch_description_from_readme, fetch_git_config, print_log,
                   render_template, set_log_config)


def main():
    args = docopt(
        __doc__, version='create-pypkg {}'.format(__version__)
    )
    set_log_config(debug=args['--debug'], info=args['--info'])
    logger = logging.getLogger(__name__)
    logger.debug('args:{0}{1}'.format(os.linesep, args))
    _create_python_package_scaffold(args=args)


def _create_python_package_scaffold(args, include_package_data=True,
                                    create_dockerfile=True):
    repo_path = Path(args['<path>']).resolve()
    package_name = args['--module'] or repo_path.name
    package_path = repo_path.joinpath(re.sub(r'[\.\-]', '', package_name))
    if not repo_path.is_dir():
        raise FileNotFoundError('No such directory: {}'.format(repo_path))
    elif not package_path.is_dir():
        print_log('Make a directory:\t{}'.format(package_path))
        package_path.mkdir()
    else:
        pass
    readme_md_path = repo_path.joinpath('README.md')
    if readme_md_path.is_file():
        description = fetch_description_from_readme(
            md_path=str(readme_md_path)
        )
    else:
        description = ''
        render_template(
            data={'package_name': package_name},
            output_path=str(readme_md_path)
        )
    data = {
        'package_name': package_name, 'module_name': package_path.name,
        'include_package_data': str(include_package_data),
        'version': 'v0.0.1', 'description': description,
        **fetch_git_config(repo_path=str(repo_path))
    }
    gitignore = repo_path.joinpath('.gitignore')
    if not gitignore.is_file():
        render_template(
            output_path=str(gitignore), template='Python.gitignore'
        )
    dest_files = [
        'setup.py',
        *[(package_path.name + '/' + n) for n in ['__init__.py', 'cli.py']],
        'MANIFEST.in', 'Dockerfile', 'docker-compose.yml'
    ]
    for f in dest_files:
        render_template(output_path=str(repo_path.joinpath(f)), data=data)
    dockerignore = repo_path.joinpath('.dockerignore')
    if not dockerignore.is_file():
        print_log('Create a symlink:\t{}'.format(dockerignore))
        os.symlink('.gitignore', str(dockerignore))



#!/usr/bin/env python

import logging
import os
import re
import shutil
from configparser import ConfigParser
from pathlib import Path

from jinja2 import Environment, FileSystemLoader


def set_log_config(debug=None, info=None):
    if debug:
        lv = logging.DEBUG
    elif info:
        lv = logging.INFO
    else:
        lv = logging.WARNING
    logging.basicConfig(
        format='%(asctime)s %(levelname)-8s %(message)s',
        datefmt='%Y-%m-%d %H:%M:%S', level=lv
    )


def fetch_description_from_readme(md_path):
    short_description = ''
    with open(md_path, 'r') as f:
        for i, r in enumerate(f):
            if i > 0 and r.strip() and not r.startswith(('#', '=', '-')):
                short_description = r.strip()
                break
    return short_description


def fetch_git_config(repo_path):
    local_gitconfig = Path(repo_path).joinpath('.git/config')
    local_cf = (
        _read_config_file(
            path=str(local_gitconfig), keys=['user']
        ) if local_gitconfig.is_file() else dict()
    )
    global_gitconfig = Path.home().joinpath('.gitconfig')
    global_cf = (
        _read_config_file(
            path=str(global_gitconfig), keys=['user', 'remote "origin"']
        ) if global_gitconfig.is_file() else dict()
    )
    if local_cf.get('user'):
        author = str(local_cf['user'].get('name'))
        author_email = str(local_cf['user'].get('email'))
    elif global_cf.get('user'):
        author = str(global_cf['user'].get('name'))
        author_email = str(global_cf['user'].get('email'))
    else:
        author = ''
        author_email = ''
    if local_cf.get('remote "origin"'):
        url = str(local_cf['remote "origin"'].get('url'))
        user_name = re.split(r'[:/]', url)[-2] if url and '/' in url else ''
    else:
        url = ''
        user_name = ''
    return {
        'author': author, 'author_email': author_email, 'url': url,
        'user_name': user_name
    }


def _read_config_file(path, keys=None):
    c = ConfigParser()
    c.read(path)
    return {k: dict(v) for k, v in c.items() if keys is None or k in keys}


def render_template(output_path, data=None, template=None):
    logger = logging.getLogger(__name__)
    if Path(output_path).is_file():
        logger.info('Skip rendering a file:\t{}'.format(output_path))
    else:
        print_log('Render a file:   \t{}'.format(output_path))
        if data is not None:
            with open(output_path, 'w') as f:
                f.write(
                    Environment(
                        loader=FileSystemLoader(
                            str(Path(__file__).parent.joinpath('../template')),
                            encoding='utf8'
                        )
                    ).get_template(
                        template or (Path(output_path).name + '.j2')
                    ).render(data) + os.linesep
                )
        else:
            shutil.copyfile(
                str(
                    Path(__file__).parent.joinpath('../static').joinpath(
                        template or Path(output_path).name
                    )
                ),
                output_path
            )


def print_log(message):
    logger = logging.getLogger(__name__)
    logger.info(message)
    print('>>\t{}'.format(message), flush=True)
